// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/cards/widgets/card_background.dart';
import 'package:wyatt_ui_kit/wyatt_ui_kit.dart';

class CardWrapper extends StatelessWidget {
  CardWrapper({
    required this.child,
    required this.backgroundColors,
    required this.borderColors,
    required this.shadow,
    required this.minSize,
    required this.maxSize,
    required this.padding,
    required this.background,
    super.key,
  });

  final Widget? background;
  final Widget child;
  final MultiColor? backgroundColors;
  final MultiColor? borderColors;
  final BoxShadow? shadow;
  final Size? minSize;
  final Size? maxSize;
  final double? padding;

  final GlobalKey _key = GlobalKey();

  Widget _buildChild(Widget child) => (background != null)
      ? Stack(
          alignment: Alignment.center,
          children: [
            CardBackground(
              cardKey: _key,
              background: background,
            ),
            Padding(
              padding: EdgeInsets.all(
                padding ?? 25,
              ),
              child: child,
            ),
          ],
        )
      : Padding(
          padding: EdgeInsets.all(
            padding ?? 25,
          ),
          child: child,
        );

  Gradient? _cardGradient(BuildContext context) {
    if (backgroundColors != null && backgroundColors!.isGradient) {
      return LinearGradientHelper.fromMultiColor(backgroundColors!);
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();

      if (extensionCardColor != null &&
          extensionCardColor.backgroundColors != null &&
          extensionCardColor.backgroundColors!.isGradient) {
        return LinearGradient(
          colors: extensionCardColor.backgroundColors!.colors,
        );
      }
    }
    return null;
  }

  Color? _cardColor(BuildContext context) {
    if (backgroundColors != null && backgroundColors!.isColor) {
      return backgroundColors!.color;
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();

      if (extensionCardColor != null &&
          extensionCardColor.backgroundColors != null) {
        return extensionCardColor.backgroundColors!.color;
      }
    }
    return Theme.of(context).cardColor;
  }

  BoxBorder? _boxBorder(BuildContext context) {
    if (borderColors != null) {
      if (borderColors!.isGradient) {
        return GradientBoxBorder(
          gradient: LinearGradientHelper.fromMultiColor(borderColors!),
        );
      } else if (borderColors!.isColor) {
        return Border.all(
          color: borderColors!.color,
        );
      }
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();
      if (extensionCardColor != null &&
          extensionCardColor.borderColors != null) {
        if (extensionCardColor.borderColors!.isGradient) {
          return GradientBoxBorder(
            gradient: LinearGradient(
              colors: extensionCardColor.borderColors!.colors,
            ),
          );
        } else if (extensionCardColor.backgroundColors!.colors.isNotEmpty) {
          return Border.all(
            color: extensionCardColor.backgroundColors!.color,
          );
        }
      }
    }
    return null;
  }

  List<BoxShadow> _shadow(BuildContext context) {
    final shadows = List<BoxShadow>.empty(growable: true);
    if (shadow != null) {
      shadows.add(shadow!);
    } else {
      final extensionCardColor =
          Theme.of(context).extension<CardThemeExtension>();
      if (extensionCardColor != null &&
          extensionCardColor.shadowColor != null) {
        shadows.add(extensionCardColor.shadowColor!);
      }
    }
    return shadows;
  }

  @override
  Widget build(BuildContext context) => SelectionArea(
        child: DecoratedBox(
          key: _key,
          decoration: BoxDecoration(
            borderRadius: const BorderRadius.all(Radius.circular(12)),
            gradient: _cardGradient(context),
            color: _cardColor(context),
            border: _boxBorder(context),
            boxShadow: _shadow(context),
          ),
          child: (minSize != null && maxSize != null)
              ? ConstrainedBox(
                  constraints: BoxConstraints(
                    minWidth: minSize!.width,
                    minHeight: minSize!.height,
                    maxWidth: maxSize!.width,
                    maxHeight: maxSize!.height,
                  ),
                  child: _buildChild(child),
                )
              : _buildChild(child),
        ),
      );
}
